package com.lcs.muscle.iogateway;

import java.lang.reflect.InvocationTargetException;

public class MessageIOGatewayFactory 
{
   // This class is a factory and shouldn't be created anywhere
   private MessageIOGatewayFactory() 
   {
     // empty
   }
   
   public static AbstractMessageIOGateway getMessageIOGateway() 
   {
      return MessageIOGatewayFactory.getMessageIOGateway(AbstractMessageIOGateway.MUSCLE_MESSAGE_DEFAULT_ENCODING);
   }
   
   public static AbstractMessageIOGateway getMessageIOGateway(int encoding) 
   {
      if (encoding == AbstractMessageIOGateway.MUSCLE_MESSAGE_DEFAULT_ENCODING) return new MessageIOGateway();

      // else, try to get the best zlib implementation currently available
      try {
         Class clazz = Class.forName("com.jcraft.jzlib.JZlib");
         // Found the JZlib library on the classpath, now return a reference to JZLibMessageIOGateway
         Class gatewayClass = Class.forName("com.lcs.muscle.iogateway.JZLibMessageIOGateway");
         return (AbstractMessageIOGateway) gatewayClass.getConstructor(new Class[] {Integer.TYPE}).newInstance(new Object[] {new Integer(encoding)});
      } catch (ClassNotFoundException e) {
         // The jcraft JZlib library is not on the classpath
         return new NativeZLibMessageIOGateway(encoding);
      } catch (InstantiationException e) {
         // The muscle JZLibMessageIOGateway can't be instantiated
         return new NativeZLibMessageIOGateway(encoding);
      } catch (IllegalAccessException e) {
         // The muscle JZLibMessageIOGateway can't be instantiated
         return new NativeZLibMessageIOGateway(encoding);
      } catch (IllegalArgumentException e) {
         return new NativeZLibMessageIOGateway(encoding);
      } catch (SecurityException e) {
         return new NativeZLibMessageIOGateway(encoding);
      } catch (InvocationTargetException e) {
         return new NativeZLibMessageIOGateway(encoding);
      } catch (NoSuchMethodException e) {
         return new NativeZLibMessageIOGateway(encoding);
      }
   }
}
